package org.anarres.jperf;

import java.util.AbstractMap;
import java.util.AbstractSet;
import java.util.Iterator;
import java.util.Set;
import java.util.Map;
import java.util.NoSuchElementException;

public abstract class PerfectMap<K,V> extends AbstractMap<K,V> {

	private Map.Entry[]	table;
	private int			size;

	public PerfectMap(Map.Entry[] table) {
		this.table = table;
		int	size = 0;
		for (int i = 0; i < table.length; i++)
			if (table[i] != null)
				size++;
		this.size = size;
	}

	/** An additional method. */
	public int getTableSize() {
		return table.length;
	}

	protected abstract int hash(K k);

	private Map.Entry<K,V> getEntry(K k) {
		int	idx = hash(k);
		if (idx < 0 || idx >= table.length)
			return null;
		Map.Entry<K,V>	e = table[idx];
		if (e == null)
			return null;
		/* XXX Requires k not null. */
		if (!k.equals(e.getKey()))
			return null;
		return e;
	}

	@Override
	public int size() {
		return size;
	}

	public boolean containsKey(Object k) {
		Map.Entry<K,V>	e = getEntry((K)k);
		return e != null;
	}

	public V get(Object k) {
		Map.Entry<K,V>	e = getEntry((K)k);
		if (e == null)
			return null;
		return e.getValue();
	}

	/* Could make put() allow replacement of values. */

	@Override
	public V remove(Object key) {
		throw new UnsupportedOperationException();
	}

	@Override
	public void clear() {
		throw new UnsupportedOperationException();
	}

	private transient Set<Map.Entry<K,V>>	entrySet = null;

	protected Iterator<Map.Entry<K,V>> entryIterator() {
		return new Iterator<Map.Entry<K,V>>() {
			private int	idx = 0;
			public boolean hasNext() {
				while (idx < table.length) {
					if (table[idx] != null)
						return true;
					idx++;
				}
				return false;
			}
			public Map.Entry<K,V> next() {
				if (!hasNext())
					throw new NoSuchElementException();
				return table[idx++];
			}
			public void remove() {
				throw new UnsupportedOperationException();
			}
		};
	}

	public Set<Map.Entry<K,V>> entrySet() {
		if (entrySet == null) {
			entrySet = new AbstractSet<Map.Entry<K,V>>() {
				public Iterator<Map.Entry<K,V>> iterator() {
					return entryIterator();
				}

				public int size() {
					return PerfectMap.this.size();
				}

				public boolean isEmpty() {
					return PerfectMap.this.isEmpty();
				}

				public void clear() {
					PerfectMap.this.clear();
				}

				public boolean contains(Object o) {
					if (!(o instanceof Map.Entry))
						return false;
					Map.Entry<K,V>	e = (Map.Entry<K,V>)o;
					Map.Entry<K,V>	f = getEntry(e.getKey());
					return (f != null) && f.equals(e);
				}

			};
		}
		return entrySet;
	}

}
