package org.anarres.jperf;

import java.util.AbstractMap;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

public class PerfectObjectMapGenerator<K,V>
					extends PerfectMapGenerator<K,V> {

	private Map<K,V>	map;

	public PerfectObjectMapGenerator() {
		this.map = new HashMap<K,V>();
	}

	public void add(K k, V v) {
		V	prev = map.put(k, v);
		if (prev != null)
			throw new IllegalArgumentException("Duplicate key " + k +
					", previous value was " + prev +
					", new value is " + v);
	}

	public int toHashCode(K key, int rem) {
		return (key.hashCode() & 0x7FFFFFFF) % rem;
	}

	public int toRemainder() {
		Collection<K>	keys = map.keySet();
		int				rem = keys.size();	/* for starters */
		FUNC: for (;;) {
			assert rem < keys.size() * 4 + 256 :
					"perfect hash failed";
			boolean[]	used = new boolean[rem];
			for (K k : keys) {
				int	hash = toHashCode(k, rem);
				if (used[hash]) {
					rem++;
					continue FUNC;
				}
				used[hash] = true;
			}

			return rem;
		}
	}

	public Map<K,V> toMap() {
		int	rem = toRemainder();
		AbstractMap.SimpleEntry[]	table =
				new AbstractMap.SimpleEntry[rem];
		for (Map.Entry<K,V> e : map.entrySet()) {
			K	k = e.getKey();
			int	hash = (k.hashCode() & 0x7FFFFFFF) % rem;
			table[hash] = new AbstractMap.SimpleEntry<K,V>(
				k, e.getValue()
			);
		}

		return new PerfectObjectMap(table, rem);
	}

}
