package org.anarres.jperf;

import java.util.*;

import junit.framework.Test;
import junit.framework.TestCase;

public class PerfectStringMapTestCase extends TestCase implements Test {

	public void testPerfectStringMap() throws Exception {
		PerfectStringMapGenerator<Integer>	gen =
				new PerfectStringMapGenerator<Integer>(
					new byte[] { 0, 1, -1 }
				);
		gen.add("", 0);
		gen.add("foo", 1);
		gen.add("bar", 2);
		gen.add("arsearsearsearse", 3);
		Map<String,Integer>	map = gen.toMap();
		assertEquals(Integer.valueOf(0), map.get(""));
		assertEquals(Integer.valueOf(1), map.get("foo"));
		assertEquals(Integer.valueOf(2), map.get("bar"));
		assertEquals(Integer.valueOf(3), map.get("arsearsearsearse"));
		assertEquals(null, map.get("other"));
		assertEquals(null, map.get("a"));
		assertEquals(null, map.get("tipple"));
		assertTrue(((PerfectMap)map).getTableSize() < 30);
		assertEquals(4, map.size());
		assertEquals(4, map.entrySet().size());
		assertTrue(!map.isEmpty());
		assertTrue(!map.entrySet().isEmpty());
		assertTrue(map.containsKey("foo"));
		assertTrue(!map.containsKey("wang"));
		assertTrue(map.entrySet().iterator().hasNext());
		assertNotNull(map.entrySet().iterator().next());

		int			i = 0;
		Iterator<?>	it = map.entrySet().iterator();
		while (it.hasNext()) {
			assertNotNull(it.next());
			i++;
			try {
				it.remove();
				fail("removed an item");
			}
			catch (UnsupportedOperationException e) {
				/* ok */
			}
		}
		assertEquals(4, i);

		try {
			it.next();
			fail("got item after last");
		}
		catch (NoSuchElementException e) {
			/* ok */
		}

		try {
			map.clear();
			fail("cleared the map");
		}
		catch (UnsupportedOperationException e) {
			/* ok */
		}

		try {
			map.entrySet().clear();
			fail("cleared the map");
		}
		catch (UnsupportedOperationException e) {
			/* ok */
		}

		assertTrue(!map.entrySet().contains("foo"));
	}

	public void testEmptyStringMap() throws Exception {
		PerfectStringMapGenerator<Class>	gen =
				new PerfectStringMapGenerator<Class>(
					new byte[] { 1, 2 }
				);
		Map<String,Class>	map = gen.toMap();
		assertTrue(map.isEmpty());
		assertEquals(0, map.size());
		assertTrue(!map.entrySet().iterator().hasNext());
	}

}
